"""Module to run tb3py."""

import os
import numpy as np
from julia.api import Julia
from jarvis.core.kpoints import Kpoints3D as Kpoints
import matplotlib.pyplot as plt
from jarvis.db.figshare import get_jid_data
from jarvis.core.atoms import Atoms
import argparse
import sys

plt.switch_backend("agg")

angst_to_bohr = 1  # 1.88973
const = 13.605662285137


sysimage = os.path.join(
    os.environ["HOME"], ".julia", "sysimages", "sys_threebodytb.so"
)
julia_cmd = mpath = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    "julia",
    "julia-1.6.1",
    "bin",
    "julia",
)
julia_bin = os.path.join(
    os.path.dirname(os.path.realpath(__file__)),
    "tb3py",
    "julia",
    "julia-1.6.1",
    "bin",
)  # mpath+"/src/julia/julia-1.6.1/bin/julia" # path for julia
os.environ["PATH"] += os.pathsep + os.path.join(julia_bin)
# print(os.environ["PATH"])
jlsession = Julia(runtime=julia_cmd, compiled_modules=False, sysimage=sysimage)
jlsession.eval("using Suppressor")  # suppress output
try:
    from julia import ThreeBodyTB as TB
except Exception:
    pass


def get_crys_from_atoms(atoms=None):
    """Get TightlyBound crys object from jarvis.core.Atoms."""
    lattice_mat = np.array(atoms.lattice_mat, dtype="float") * angst_to_bohr
    frac_coords = np.array(atoms.frac_coords, dtype="float")
    elements = np.array(atoms.elements)
    crys = TB.makecrys(lattice_mat, frac_coords, elements)
    return crys


def get_energy_force_stress(atoms=None):
    """Get energy, forces and strss."""
    crys = get_crys_from_atoms(atoms)
    energy, f_cart, stress, tbc = TB.scf_energy_force_stress(crys)
    print("energy, f_cart, stress", energy, f_cart, stress)
    return energy, f_cart, stress, tbc


def get_energy_bandstructure(atoms=None, filename=None):
    """Get bandstructure, total energy and bandgap."""
    print(atoms)
    kpoints = Kpoints().kpath(atoms, line_density=20)
    labels = kpoints.to_dict()["labels"]
    kpts = kpoints.to_dict()["kpoints"]
    crys = get_crys_from_atoms(atoms)
    energy, tbc, flag = TB.scf_energy(crys)
    tot_energy = round(const * float(energy), 5)
    vals = const * np.array(TB.calc_bands(tbc, kpts) - tbc.efermi)
    vbm, cbm = TB.TB.find_vbm_cbm(vals, const * tbc.efermi)
    band_gap = cbm - vbm
    if band_gap < 0:
        band_gap = 0
    if filename is not None:
        for i, j in enumerate(vals.T):
            plt.plot(j, c="blue")
        kp = np.arange(len(vals))
        new_kp = []
        new_labels = []
        count = 0

        for i, j in zip(kp, labels):
            if j != "":
                if count > 1 and count < len(labels) - 1:
                    if labels[count] != labels[count + 1]:
                        new_kp.append(i)
                        new_labels.append("$" + str(j) + "$")
                else:
                    new_kp.append(i)
                    new_labels.append("$" + str(j) + "$")
            count += 1
        plt.rcParams.update({"font.size": 18})
        plt.xticks(new_kp, new_labels)
        plt.axhline(y=0, c="g", linestyle="-.")
        plt.ylabel("Energy (eV)")
        plt.tight_layout()
        plt.savefig(filename)
        plt.close()
        print("Bandstructure plot saved in:", filename)
    return tot_energy, band_gap, tbc


def example():
    """Get an example."""
    A = np.eye(3) * 5.0
    coords = np.zeros((1, 3))
    c = TB.makecrys(A, coords, ["Li"])
    energy, tbc, flag = TB.scf_energy(c)
    return energy, tbc, flag


def predict_for_poscar(filename="POSCAR", band_file="bands.png"):
    """Predict properties for a POSCAR file."""
    atoms = Atoms.from_poscar(filename)
    tot_energy, band_gap, tbc = get_energy_bandstructure(
        atoms=atoms, filename=band_file
    )
    print("tot_energy, band_gap", tot_energy, band_gap)
    return tot_energy, band_gap


def predict_for_cif(filename="atoms.cif", band_file="bands.png"):
    """Predict properties for a .cif file."""
    try:
        atoms = Atoms.from_cif(filename)
    except Exception as exp:
        raise NotImplementedError("pip install cif2cell, and try again", exp)
        pass
    tot_energy, band_gap, tbc = get_energy_bandstructure(
        atoms=atoms, filename=band_file
    )
    print("tot_energy, band_gap", tot_energy, band_gap)
    return tot_energy, band_gap


def example_from_jarvis_db():
    """Run an example calculation with structure from JARVIS-DFT."""
    atoms = Atoms.from_dict(
        get_jid_data(jid="JVASP-1002", dataset="dft_3d")["atoms"]
    )
    print(atoms)
    tot_energy, band_gap, tbc = get_energy_bandstructure(
        atoms=atoms, filename="bands.png"
    )
    print("tot_energy, band_gap", tot_energy, band_gap)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Python wrapper for Tight-binding two and three body."
    )

    parser.add_argument(
        "--poscar_file",
        default="NA",
        help="Path to a POSCAR file.",
    )

    parser.add_argument(
        "--cif_file",
        default="NA",
        help="Path to a .cif file.",
    )

    args = parser.parse_args(sys.argv[1:])
    if args.poscar_file != "NA":
        predict_for_poscar(filename=args.poscar_file)
    elif args.cif_file != "NA":
        predict_for_cif(filename=args.cif_file)
    else:
        raise NotImplementedError(args)



#!/usr/bin/python3
import os
import sys
import subprocess

# import requests
import glob
import os

from setuptools import setup, find_packages

# TB3_DIR = os.path.dirname(os.path.abspath(__file__))

base_dir = os.path.dirname(__file__)
with open(os.path.join(base_dir, "README.md")) as f:
    long_d = f.read()


def precompile(julia=None):
    """Precompile Julia"""
    print("precompile")
    if julia is None:
        julia = "julia"  # julia command
    #    os.system(julia+" --eval  \"using TightlyBound; TightlyBound.compile()\"")
    os.system(
        julia
        + ' --eval  "using ThreeBodyTB; using Plots; ThreeBodyTB.compile()"'
    )


def install(julia=None):
    """Install Julia and TB"""
    if julia is None:
        julia = "julia"  # julia command
    print("install ", julia)
    #    os.system(julia+" --eval "+"\"import Pkg; Pkg.add(url=\\\"https://github.com/kfgarrity/ThreeBodyTB.jl\\\")\"")

    os.system(julia + " --eval " + '"import Pkg; Pkg.add(\\"ThreeBodyTB\\")"')
    os.system(julia + " --eval " + '"import Pkg; Pkg.add(\\"PyCall\\")"')
    os.system(julia + " --eval " + '"import Pkg; Pkg.add(\\"Plots\\")"')
    os.system(julia + " --eval " + '"import Pkg; Pkg.add(\\"Suppressor\\")"')
    precompile(julia)


setup(
    name="tb3py",
    version="2021.07.11",
    long_description=long_d,
    install_requires=[
        "requests>=2.26.0",
        "numpy>=1.19.5",
        "julia>=0.5.6",
        "jarvis-tools",
        "pytest",
    ],
    extras_require={
        "jarvis-tools": ["jarvis-tools"],
    },
    author="Kevin Garrity, Kamal Choudhary",
    author_email="kevin.garrity@nist.gov",
    description=(
        "tb3: an open-source software package for accurate and efficient electronic structure calculations using tight-binding (TB), including three-body interactions. https://pages.nist.gov/ThreeBodyTB.jl"
    ),
    license="NIST",
    url="https://github.com/usnistgov/tb3py",
    packages=find_packages(),
    # long_description=open(os.path.join(os.path.dirname(__file__), "README.rst")).read(),
    classifiers=[
        "Programming Language :: Python :: 3.7",
        "Development Status :: 4 - Beta",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering",
    ],
    # scripts=glob.glob(os.path.join(JARVIS_DIR,  "*"))
)


mpath = os.path.dirname(os.path.realpath(__file__))


# main path
try:
    out, err = subprocess.Popen(
        ["julia", "--version"],
        stdout=subprocess.PIPE,
        stderr=subprocess.STDOUT,
    ).communicate()
except:
    out = ""
# check if JUlia has the correct version
hasjulia = (
    ("julia version 1.6" in str(out))
    or ("julia version 1.5" in str(out))
    or ("julia version 1.7" in str(out))
    or ("julia version 1.8" in str(out))
)


# print(hasjulia) ; exit()

print("hasjulia: ", hasjulia)

if not hasjulia:  # if the correct Julia version is not present
    julia_cmd = os.path.join(
        mpath, "tb3py", "julia", "julia-1.6.1", "bin", "julia"
    )  # mpath+"/src/julia/julia-1.6.1/bin/julia" # path for julia
    # julia_cmd = mpath+"/src/julia/julia-1.6.1/bin/julia" # path for julia
    if not os.path.exists(julia_cmd):
        print("Julia not present in path:", mpath, "...downloading...")
        subfolder = os.path.join(mpath, "tb3py", "julia")
        if not os.path.exists(subfolder):
            os.makedirs(subfolder)
        # os.system("mkdir "+mpath+"/src/julia") # create a subfodler for julia
        os.chdir(subfolder)  # go to the subfolder
        # os.chdir(mpath+"/src/julia") # go to the subfolder
        ## download julia
        juliafile = os.path.join(
            mpath, "julia-1.6.1-linux-x86_64.tar.gz"
        )  # julia file
        # juliafile = os.path.join(mpath,"julia-1.6.1-linux-x86_64.tar.gz")  # julia file
        # j_link = "https://julialang-s3.julialang.org/bin/linux/x64/1.6/"
        j_link = (
            "https://julialang-s3.julialang.org/bin/linux/x64/1.6/"
            + "julia-1.6.1-linux-x86_64.tar.gz"
        )
        print("jlink", j_link)
        import requests
        import tarfile

        r_julia = requests.get(j_link, stream=True).content
        with open(juliafile, "wb") as jfile:
            jfile.write(r_julia)
        print("juliafile", juliafile)
        try:
            my_tar = tarfile.open(juliafile)
            my_tar.extractall()
            my_tar.close()
        except Exception as exp:
            print(exp)
            cmd = "tar -xvzf " + juliafile
            os.system(cmd)
        os.remove(juliafile)
        # os.system("wget https://julialang-s3.julialang.org/bin/linux/x64/1.6/"+juliafile)
        # os.system("tar -xvf "+juliafile) # untar the file
        # os.system("rm "+juliafile) # rm the file
    else:
        print("We already downloaded to ", julia_cmd)

    # julia_cmd = mpath+"/src/julia/julia-1.6.1/bin/julia" # path for julia
    os.chdir(mpath)  # go back

else:
    julia_cmd = "julia"
    print("Correct Julia version found in path")


print("My julia command : ", julia_cmd)

# install python dependences
# os.system("python3 -m pip install --user julia")

# using julia
# import julia
# julia.install()


# os.system("pip install pmdarima")


# install
sysimage = os.path.join(
    os.environ["HOME"], ".julia", "sysimages", "sys_threebodytb.so"
)
print("sysimage", sysimage)
if not os.path.isfile(sysimage):
    # import TB3.juliarun as juliarun
    install(julia_cmd)  # install Julia dependences
julia_bin = os.path.join(
    mpath, "tb3py", "julia", "julia-1.6.1", "bin"
)  # mpath+"/src/julia/julia-1.6.1/bin/julia" # path for julia
os.environ["PATH"] += os.pathsep + os.path.join(julia_bin)
"""

from julia.api import Julia
jlsession = Julia(runtime=julia_cmd, compiled_modules=False, sysimage=sysimage)
jlsession.eval("using Suppressor") # suppress output

from julia import ThreeBodyTB
c = ThreeBodyTB.makecrys( [[5.0,0,0],[0,5.0,0],[0,0,5.0]] , [[0, 0, 0]], ["Li"])

print()
print("result of makecrys")
print(c)
print()
print("the end")
print()
"""



from tb3py.main import example, get_energy_bandstructure


def test_example():
    energy, tbc, flag = example()
    print(energy)
    assert round(energy, 5) == round(-0.43160184930052803, 5)


def test_jarvis_dat():
    from jarvis.db.figshare import get_jid_data
    from jarvis.core.atoms import Atoms

    atoms = Atoms.from_dict(
        get_jid_data(jid="JVASP-664", dataset="dft_2d")["atoms"]
    )
    get_energy_bandstructure(atoms=atoms)


# test_example()
